/*****************************************************************************
******************************************************************************
*
*               Document class for I/O operations
*
*               $Revision: 1.1.2.6 $
*               $Date: 2011/06/01 14:15:01 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/

// Copyright (C) 1998  Chris Morley
// Modified by HUMUSOFT s.r.o.  Copyright 1998-2011 HUMUSOFT s.r.o.

//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef DOC_H
#define DOC_H

#include "System.h"

#include <fstream>
#include <stdio.h>
#include <iostream>
#include <string.h>


class OPENVRMLDLLEXPORT Doc
{
friend class DocInputStream;

private:

  char* d_url;
  char* d_tmpfile;   // Local copy of http: files
  FILE *d_fp;


public:
  Doc(const char *url = 0, const Doc* relative = 0);
  Doc(const Doc*);
  ~Doc();

  FILE *fopen(const char *mode);
  void fclose();


public:

  static const char *stripProtocol(const char *url);
  static bool isAbsolute(const char *url);

  const char* localName();  // "/tmp/file.xyz" or NULL
  const char* localPath();  // "/tmp/" or NULL

  bool filename(char * fn, int nfn);

  const char *url() const;          // "http://www.foo.com/dir/file.xyz#Viewpoint"
  const char *urlBase() const;      // "file" or ""
  const char *urlExt() const;       // "xyz" or ""
  const char *urlPath() const;      // "http://www.foo.com/dir/" or ""
  char const* urlProtocol() const; // "http"
  const char *urlModifier() const;  // "#Viewpoint" or ""

  void seturl(const char* url, const Doc* relative = 0);

  Doc(const Doc& other);
  Doc& operator=(const Doc& other);
};


class DocInputStream : public std::ifstream
{
public:
  DocInputStream(Doc& doc) : std::ifstream(doc.localName())
  { }
};


#endif // DOC_H
