/*****************************************************************************
******************************************************************************
*
*               Container for objects that can be referenced by handle
*
*               $Revision: 1.1.2.1 $
*               $Date: 2011/07/19 14:52:24 $
*               $Author: Xjhouska $
*
*               Copyright 1998-2011 HUMUSOFT s.r.o.
*
******************************************************************************
*****************************************************************************/


#ifndef HANDLEFACTORY_H
#define HANDLEFACTORY_H


#include <map>


template<typename T> class HandleFactory : public std::map<unsigned, T*>
{
private:

unsigned next;

public:


 /* constructor */
 HandleFactory(void) : next(1)
 { };


/* insert a newly created object into container, creating an unique handle for it */
/* 0 is the invalid handle value */
unsigned add(T* object)
{
  unsigned handle = next;
  this->operator[](handle) = object;
  do next++; while (next==0 || this->find(next)!=this->end());
  return(handle);
}


/* get object by its handle, NULL if not there */
T* get(unsigned handle) const
{
  typename std::map<unsigned, T*>::const_iterator it = this->find(handle);
  return((it != this->end()) ? it->second : NULL);
}


/* get handle by object pointer, 0 if not there */
unsigned gethandle(T* value) const
{
  typename std::map<unsigned, T*>::const_iterator it;
  for (it=this->begin(); it!=this->end(); it++)
    if (value == it->second)
      return(it->first);

  return(0);
}


/* get all handles in container, store into array of unsigned */
void getallhandles(size_t maxsize, unsigned* array) const
{
  typename std::map<unsigned, T*>::const_iterator it;
  size_t i;
  for (i=0, it=this->begin(); i<maxsize && it!=this->end(); ++i, ++it)
    *array++ = it->first;
}


/* get all handles in container, store into array of double */
void getallhandles(size_t maxsize, double* array) const
{
  typename std::map<unsigned, T*>::const_iterator it;
  size_t i;
  for (i=0, it=this->begin(); i<maxsize && it!=this->end(); ++i, ++it)
    *array++ = it->first;
}


/* delete an object by handle, optionally remove it from container */
bool destroy(unsigned handle, bool eraseit=true)
{
  typename std::map<unsigned, T*>::iterator it = this->find(handle);
  if (it == this->end()) return(false);

  delete(it->second);
  if (eraseit) this->erase(it);
  return(true);
}


/* delete all objects in container, clear the container */
void destroyall(void)
{
  typename std::map<unsigned, T*>::iterator it;
  for (it=this->begin(); it!=this->end(); it++)
    delete(it->second);
  this->clear();
}


/* destructor - same as destroyall */
 ~HandleFactory(void)
 { destroyall(); };


};



#endif
