/*****************************************************************************
******************************************************************************
*
*               VRML Cone node
*
*               $Revision: 1.1.2.2 $
*               $Date: 2009/04/15 13:16:45 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/

// Copyright (C) 1998  Chris Morley
// Modified by HUMUSOFT s.r.o.  Copyright 1998-2009 HUMUSOFT s.r.o.

//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#include "VrmlNodeCone.h"

#include "VrmlNodeType.h"
#include "Viewer.h"


//  Return a new VrmlNodeCone
static VrmlNode *creator( VrmlScene *s ) { return new VrmlNodeCone(s); }


// Define the built in VrmlNodeType:: "Cone" fields

VrmlNodeType *VrmlNodeCone::defineType(VrmlNodeType *t)
{
  static VrmlNodeType *st = 0;

  if (! t)
    {
      if (st) return st;	// Only define type once.
      t = st = new VrmlNodeType("Cone", creator);
      t->reference();
    }

  VrmlNodeGeometry::defineType(t);	// Parent class
  t->addField("bottom", VrmlField::SFBOOL, VrmlSFBool(true));
  t->addField("bottomRadius", VrmlField::SFFLOAT, VrmlSFFloat(1.0));
  t->addField("height", VrmlField::SFFLOAT, VrmlSFFloat(2.0));
  t->addField("side", VrmlField::SFBOOL, VrmlSFBool(true));

  return t;
}

VrmlNodeType & VrmlNodeCone::nodeType() const
{
    return *defineType(0);
}


VrmlNodeCone::VrmlNodeCone(VrmlScene *scene) :
  VrmlNodeGeometry(scene),
  d_bottom(true),
  d_bottomRadius(1.0),
  d_height(2.0),
  d_side(true)
{
}

VrmlNodeCone::~VrmlNodeCone()
{
}


VrmlNode *VrmlNodeCone::cloneMe() const
{
  return new VrmlNodeCone(*this);
}


ostream& VrmlNodeCone::printFields(ostream& os, int indent)
{
  if (! d_bottom.get()) PRINT_FIELD(bottom);
  if (d_bottomRadius.get() != 1.0) PRINT_FIELD(bottomRadius);
  if (d_height.get() != 2.0) PRINT_FIELD(height);
  if (! d_side.get()) PRINT_FIELD(side);

  return os;
}

Viewer::Object VrmlNodeCone::insertGeometry(Viewer *viewer, VrmlRenderContext rc)
{
  const char* nodename = (*(name()) == '\0') ? "(unnamed Cone)" : name();

  float height = d_height.get();
  if (height<0)
    theSystem->warn(System::SEVERITY_LOW, System::WARNINGTYPE_UNKNOWN,
      "Invalid value for %s.height: %f (nonnegative number is required)",
      nodename, height);

  float radius = d_bottomRadius.get();
  if (radius<0)
    theSystem->warn(System::SEVERITY_LOW, System::WARNINGTYPE_UNKNOWN,
      "Invalid value for %s.bottomRadius: %f (nonnegative number is required)",
      nodename, radius);

  return viewer->insertCone(height,
			    radius,
			    d_bottom.get(),
			    d_side.get());
}

#if defined(__ORBISNAP__)

const VrmlField *VrmlNodeCone::getField(const char *fieldName) const
{
  if (strcmp(fieldName, "bottomRadius") == 0)
    return &d_bottomRadius;
  else if (strcmp(fieldName, "height") == 0)
    return &d_height;
  else if (strcmp(fieldName, "side") == 0)
    return &d_side;
  else if (strcmp(fieldName, "bottom") == 0)
    return &d_bottom;
  return VrmlNodeGeometry::getField(fieldName);
}

#endif

// Set the value of one of the node fields.

void VrmlNodeCone::setField(const char *fieldName,
			    const VrmlField &fieldValue)
{
  if TRY_FIELD(bottom, SFBool)
  else if TRY_FIELD(bottomRadius, SFFloat)
  else if TRY_FIELD(height, SFFloat)
  else if TRY_FIELD(side, SFBool)
  else
    VrmlNodeGeometry::setField(fieldName, fieldValue);
}
