/*****************************************************************************
******************************************************************************
*
*               VRML DirectionalLight node
*
*               $Revision: 1.1.2.2 $
*               $Date: 2009/04/15 13:16:45 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/

// Copyright (C) 1998  Chris Morley
// Modified by HUMUSOFT s.r.o.  Copyright 1998-2009 HUMUSOFT s.r.o.

//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#include "VrmlNodeDirLight.h"
#include "VrmlNodeType.h"
#include "MathUtils.h"
#include "Viewer.h"

// Return a new VrmlNodeDirLight
static VrmlNode *creator( VrmlScene *s ) { return new VrmlNodeDirLight(s); }


// Define the built in VrmlNodeType:: "DirLight" fields

VrmlNodeType *VrmlNodeDirLight::defineType(VrmlNodeType *t)
{
  static VrmlNodeType *st = 0;

  if (! t)
    {
      if (st) return st;
      t = st = new VrmlNodeType("DirectionalLight", creator);
      t->reference();
    }

  // VrmlNodeLight::defineType(t);	// Parent class
  // dont' call parent in this case. Initiate them in this class

  t->addExposedField("ambientIntensity", VrmlField::SFFLOAT,VrmlSFFloat(0.0));
  t->addExposedField("color", VrmlField::SFCOLOR, VrmlSFColor(1, 1, 1));
  t->addExposedField("direction", VrmlField::SFVEC3F, VrmlSFVec3f(0, 0, -1));
  t->addExposedField("intensity", VrmlField::SFFLOAT, VrmlSFFloat(1));
  t->addExposedField("on", VrmlField::SFBOOL, VrmlSFBool(true));

  return t;
}

VrmlNodeType & VrmlNodeDirLight::nodeType() const
{
    return *defineType(0);
}


VrmlNodeDirLight::VrmlNodeDirLight(VrmlScene *scene) :
  VrmlNodeLight(scene),
  d_direction(0.0, 0.0, -1.0)
{
}

VrmlNodeDirLight::~VrmlNodeDirLight()
{
}


VrmlNode *VrmlNodeDirLight::cloneMe() const
{
  return new VrmlNodeDirLight(*this);
}


ostream& VrmlNodeDirLight::printFields(ostream& os, int indent)
{
  VrmlNodeLight::printFields(os, indent);
  if (! FPZERO(d_direction.getX()) ||
      ! FPZERO(d_direction.getY()) ||
      ! FPEQUAL(d_direction.getZ(), -1.0) )
    PRINT_FIELD(direction);

  return os;
}

// This should be called before rendering any sibling nodes.

void VrmlNodeDirLight::render(Viewer *viewer, VrmlRenderContext rc)
{
  if (d_on.get() )
    d_viewerObject = viewer->insertDirLight( d_ambientIntensity.get(),
			    d_intensity.get(),
			    d_color.get(),
			    d_direction.get() );

  clearModified();
}

// Get the value of one of the exposedFields

const VrmlField *VrmlNodeDirLight::getField(const char *fieldName) const
{
  if ( strcmp( fieldName, "direction" ) == 0 )
    return &d_direction;
  return VrmlNodeLight::getField( fieldName );
}

// Set the value of one of the node fields.

void VrmlNodeDirLight::setField(const char *fieldName,
				const VrmlField &fieldValue)
{
  if TRY_FIELD(direction, SFVec3f)
  else
    VrmlNodeLight::setField(fieldName, fieldValue);
}

const VrmlSFVec3f& VrmlNodeDirLight::getDirection() const   // LarryD Mar 04/99
{  return d_direction; }
