/*****************************************************************************
******************************************************************************
*
*               VRML Group node
*
*               $Revision: 1.1.2.3 $
*               $Date: 2011/07/19 14:53:20 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/

// Copyright (C) 1998  Chris Morley
// Modified by HUMUSOFT s.r.o.  Copyright 1998-2011 HUMUSOFT s.r.o.

//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef  _VRMLNODEGROUP_
#define  _VRMLNODEGROUP_

#include "VrmlField.h"
#include "VrmlBSphere.h"
#include "VrmlNodeChild.h"
#include "Viewer.h"

/**
 * VrmlNodeGroup
 */
class VrmlNodeGroup : public VrmlNodeChild {

public:

  // Define the fields of all built in group nodes
  static VrmlNodeType *defineType(VrmlNodeType *t = 0);
  virtual VrmlNodeType & nodeType() const;

  VrmlNodeGroup(VrmlScene *s = 0);
  virtual ~VrmlNodeGroup();

  virtual VrmlNode *cloneMe() const;
  virtual void cloneChildren(VrmlNamespace *);

  virtual bool isModified() const;
  virtual void updateModified(VrmlNodePath& path, int flags);
  virtual void clearFlags();

  virtual void addToScene( VrmlScene *s, const char *relativeUrl );

  virtual ostream& printFields(ostream& os, int indent);

  virtual void render(Viewer *, VrmlRenderContext rc);
  virtual void renderNoCull(Viewer *, VrmlRenderContext rc);

  virtual void accumulateTransform(VrmlNode*);

  void activate( double timeStamp, bool isOver, bool isActive, double *p );

  void addChildren( const VrmlMFNode &children );
  void removeChildren( const VrmlMFNode &children );
  void removeChildren();

  virtual void setEncapsulatingPrototype(VrmlNodeType* type);

  virtual void setProto(VrmlNodeProto *proto);

  virtual void eventIn(double timeStamp,
		       const char *eventName,
		       const VrmlField *fieldValue);

  int size();
  VrmlNode *child(int index);

  virtual VrmlNode* getParentTransform();

  // LarryD Feb 11/99
  VrmlMFNode *getNodes()  { return &d_children;}

  const VrmlBVolume* getBVolume() const;

 /**
  * Guess znear and zfar values
  */
  void guessNearFarPlane(Viewer* viewer, double* nearVal, double* farVal);

  void invalidate();

protected:

  VrmlField* findField(const char* fieldname);
  void onFieldChange(const char* fieldname, const VrmlField &value);

  VrmlSFVec3f d_bboxCenter;
  VrmlSFVec3f d_bboxSize;
  VrmlMFNode d_children;

  VrmlSFString d_relative;
  VrmlNode *d_parentTransform;
  Viewer::Object d_viewerObject;

  /**
   * Cached copy of the bsphere enclosing this node's children.
   */
  VrmlBSphere d_bsphere;

  /**
   * Construct a bounding sphere around this node's children. Store it
   * in d_bsphere.
   */
  virtual void recalcBSphere();
};

#endif // _VRMLNODEGROUP_

