/*****************************************************************************
******************************************************************************
*
*               VRML IndexedLineSet node
*
*               $Revision: 1.1.2.5 $
*               $Date: 2010/03/16 08:40:10 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/

// Copyright (C) 1998  Chris Morley
// Modified by HUMUSOFT s.r.o.  Copyright 1998-2010 HUMUSOFT s.r.o.

//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#include "VrmlNodeILineSet.h"
#include "VrmlNodeType.h"
#include "VrmlNodeColor.h"
#include "VrmlNodeCoordinate.h"
#include "VrmlNodeTextureCoordinate.h"
#include "VrmlNodeProto.h"
#include "Viewer.h"

static VrmlNode *creator( VrmlScene *s ) { return new VrmlNodeILineSet(s); }


// Define the built in VrmlNodeType:: "IndexedLineSet" fields

VrmlNodeType *VrmlNodeILineSet::defineType(VrmlNodeType *t)
{
  static VrmlNodeType *st = 0;

  if (! t)
    {
      if (st) return st;
      t = st = new VrmlNodeType("IndexedLineSet", creator);
      t->reference();
    }

  VrmlNodeIndexedSet::defineType(t);	// Parent class

  return t;
}

VrmlNodeType & VrmlNodeILineSet::nodeType() const
{
    return *defineType(0);
}

VrmlNodeILineSet::VrmlNodeILineSet(VrmlScene *scene) :
  VrmlNodeIndexedSet(scene)
{
}

VrmlNodeILineSet::~VrmlNodeILineSet()
{
}


VrmlNode *VrmlNodeILineSet::cloneMe() const
{
  return new VrmlNodeILineSet(*this);
}

void VrmlNodeILineSet::cloneChildren(VrmlNamespace* ns)
{
  if (d_color.get())
    d_color.set(d_color.get()->clone(ns));
  if (d_coord.get())
    d_coord.set(d_coord.get()->clone(ns));
}


// TO DO colors

Viewer::Object VrmlNodeILineSet::insertGeometry(Viewer *viewer, VrmlRenderContext rc)
{
  long i = 0;
  long nLines = 0;
  long maxCoordIndex  = -1;

  Viewer::Object obj = 0;

  const long* coordIndices = d_coordIndex.get();
  long nCoordIndices = d_coordIndex.getLength();
  const long* colorIndices = d_colorIndex.get();
  long nColorIndices = d_colorIndex.getLength();

  VrmlNodeCoordinate* coordNode = NodeDynamicCast<VrmlNodeCoordinate*>(d_coord.get());
  VrmlMFVec3f emptycoord;
  VrmlMFVec3f &coord = coordNode ? coordNode->coordinate() : emptycoord;
  VrmlNodeColor* colorNode = NodeDynamicCast<VrmlNodeColor*>(d_color.get());
  const float* colors = colorNode ? colorNode->color().get() : 0;
  long nVertices  = coordNode    ? coordNode->coordinate().getLength() : 0;
  long nColors    = colorNode    ? colorNode->color().getLength() : 0;

  // determine the node name to be printed in warnings
  const char* nodename = (*(name()) == '\0') ? "(unnamed IndexedLineSet)" : name();

  //:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  // check coord indices and count lines
  //:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

  for (i=0; i<nCoordIndices; i++)
  {
    if (coordIndices[i] >= nVertices || coordIndices[i] < -1)
    {
      theSystem->warn(System::SEVERITY_LOW, System::WARNINGTYPE_UNKNOWN,
        "Invalid value for %s.coordIndex(%d): %d (max is %d)",
        nodename, i, coordIndices[i], nVertices - 1);

      isValid = false;
      return(obj);
    }

    // new line occured
    if (coordIndices[i] == -1) nLines++;
    // remember the highest coord index
    if (maxCoordIndex < coordIndices[i]) maxCoordIndex = coordIndices[i];
  }

  // last line does not have to have "-1" separator on the end
  if (nCoordIndices > 0 && coordIndices[i-1] != -1) nLines++;

  checkIndexSetSemantics(nodename,
                       coordIndices,
                       nCoordIndices,
                       colorIndices,
                       nColorIndices,
                       nVertices,
		       colors,
                       nColors,
                       nLines,
                       maxCoordIndex);

  obj =  viewer->insertLineSet(nVertices,
                               coord[0],
                               d_coordIndex.getLength(),
                               d_coordIndex.get(),
                               d_colorPerVertex.get(),
                               colors,
                               nColorIndices,
                               colorIndices);


  if (d_color.get()) d_color.get()->clearModified();
  if (d_coord.get()) d_coord.get()->clearModified();

  return obj;
}
