/*****************************************************************************
******************************************************************************
*
*               VRML Proto node
*
*               $Revision: 1.1.2.8 $
*               $Date: 2011/12/12 20:17:00 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/

// Copyright 1998-2011 HUMUSOFT s.r.o.
// Portions Copyright (C) 1998  Chris Morley

//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef VRMLNODEPROTO_H
#define VRMLNODEPROTO_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "VrmlNode.h"
#include "VrmlNodeType.h"
#include "Viewer.h"

class VrmlMFNode;

class VrmlNodeProto : public VrmlNode {

#if defined(__ORBISNAP__)

  friend class VrmlNode;

#endif

public:

  virtual VrmlNodeType & nodeType() const;

  VrmlNodeProto(VrmlNodeType *nodeDef, VrmlScene *scene);
  VrmlNodeProto(const VrmlNodeProto&);
  virtual ~VrmlNodeProto();

  virtual VrmlNode *cloneMe() const;
  virtual void cloneChildren(VrmlNamespace *);

  // method used when instantiating contents of VrmlNodeProto
  virtual VrmlNode* instantiate(VrmlNamespace*);

  // method used for PROTO reinstantiation
  virtual void reinstantiate();

  virtual void addToScene( VrmlScene *, const char *relUrl );
  virtual ostream& printFields(ostream& os, int indent);

  virtual void render(Viewer *, VrmlRenderContext rc);

  virtual void eventIn(double timeStamp,
		       const char *eventName,
		       const VrmlField *fieldValue);

  virtual const VrmlField *getField(const char *fieldName) const;
  virtual void setField(const char *fieldName, const VrmlField &fieldValue);

  virtual void PROTOFieldChanged(const char *fieldName);

  virtual void updateModified(VrmlNodePath& path, int flags);

  virtual void clearModifiedAll();

  virtual void accumulateTransform( VrmlNode* );

  virtual void setEncapsulatingPrototype(VrmlNodeType* type);

  virtual void setProto(VrmlNodeProto *proto);

  void addImplementationNodesToScene(const char *relUrl = 0);

  // LarryD  Feb 11/99
  int size();
  // LarryD  Feb 11/99
  VrmlNode *child(int index);

  // LarryD Feb 11/99
  VrmlMFNode *getNodes()  { return d_nodes;}

 /**
  * Guess znear and zfar values
  */
  void guessNearFarPlane(Viewer* viewer, double* nearVal, double* farVal);

  // Field name/value pairs specified in PROTO instantiation
  typedef struct {
    char *name;
    VrmlField *value;
  } NameValueRec;

  const VrmlBVolume* getBVolume() const;

  VrmlNode *firstNode() const;

  void invalidate();

  // Find a field by name
  NameValueRec *findField(const char *fieldName) const;

  bool isInstantiated() { return d_instantiated; }

private:

  // Instantiate the proto by cloning the node type implementation nodes.
  void instantiate();

  VrmlNodeType *d_nodeType;	// Definition

  bool d_instantiated;
  VrmlNamespace *d_scope;	// Node type and name bindings

  VrmlMFNode *d_nodes;		// Local copy of implementation nodes.

  std::list<NameValueRec*> d_fields;	// Field values

  // Dispatch eventIns from outside the PROTO to internal eventIns
  typedef struct {
    char *name;
    VrmlNodeType::ISMap ismap;
  } EventDispatch;

  typedef std::list<EventDispatch*> EventDispatchList;

  EventDispatchList d_eventDispatch;

  Viewer::Object d_viewerObject; // move to VrmlNode.h ? ...

};


// Template to cast nodes, taking PROTOs and their first nodes into account
// Must be here because it needs VrmlNodeProto to be defined
template<typename T> T NodeDynamicCast(VrmlNode* n)
{
  T node = dynamic_cast<T>(n);
  if (node) return(node);        // it is a node of required type

  VrmlNodeProto* pnode = dynamic_cast<VrmlNodeProto*>(n);	 // it's a PROTO - try casting its first node
  return((pnode && pnode->firstNode()) ? dynamic_cast<T>(pnode->firstNode()) : 0);
}



#endif // VRMLNODEPROTO_H
