/*****************************************************************************
******************************************************************************
*
*               Ancestor for other VRML entities
*               which allows setting, getting and printing of comment
*
*               $Revision: 1.1.2.3 $
*               $Date: 2011/07/26 12:42:23 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/

// Copyright 1998-2011 HUMUSOFT s.r.o.

//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef VRML_OBJECT_H
#define VRML_OBJECT_H

#include "System.h"
#include <string>
#include <sstream>


class OPENVRMLDLLEXPORT VrmlObject {

private:

  /** text description for node -- used to generate comments in Vrml file */
  std::string d_comment;


public:

  /** Constructor */
  VrmlObject() {};

  /** Copy constructor */
  VrmlObject(const VrmlObject &src)
  {
    d_comment = src.d_comment;
  }

  /** sets the contents of the suplied object */
  bool set(const VrmlObject &src)
  {
    d_comment = src.d_comment;
    return true;
  }

  /**
   * setter and getter for object comment
   */
  void setComment(const std::string& comment)
  {
    d_comment = comment;
  }

  const std::string& getComment() const
  {
    return d_comment;
  }

  /** append supplied string to comments */
  void appendComment(const std::string& comment)
  {
    d_comment.append(comment);
  }

  /**
    * Print comment to the output stream.
    * @param os A stream to print in.
    * @param indent A indentation.
    * @param breakLine if true newline is created before printing started
    * @return printing stream
    */
  std::ostream& printComment(std::ostream& os, int indent, bool breakLine = false) const;

  // How many spaces per indentation level when saving node
  enum { INDENT_INCREMENT = 2 };

};

#endif // VRML_OBJECT_H
