/*****************************************************************************
******************************************************************************
*
*               VRML route and route list
*
*               $Revision: 1.1.2.5 $
*               $Date: 2011/08/08 12:08:06 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/

// Copyright 1998-2011 HUMUSOFT s.r.o.
// Portions Copyright (C) 1998  Chris Morley

//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#include "VrmlRoute.h"


ostream& Route::print(ostream& os, int indent) const
{
  if (d_fromNode && !d_fromEventOut.empty() && d_toNode && !d_toEventIn.empty())
  {
    // print comment
    printComment(os, indent, true);

    // print route itself
    os << "ROUTE " << d_fromNode->name() << "." << d_fromEventOut.c_str();
    os << " TO " << d_toNode->name() << "." << d_toEventIn.c_str();
  }
  return os;
}

void RouteList::Remove(VrmlNode* fromNode, const char* fromEventOut, VrmlNode* toNode, const char* toEventIn)
{
  iterator ri;
  for (ri=begin(); ri!=end(); )
  {
    iterator r = ri++;
    if (fromNode==r->second.fromNode() &&
	strcmp(fromEventOut, r->second.fromEventOut())==0 &&
	toNode==r->second.toNode() &&
	strcmp(toEventIn, r->second.toEventIn())==0)
      erase(r);
  }
}

void RouteList::RemoveNodeNotify(VrmlNode* node)
{
  iterator ri;
  for (ri=begin(); ri!=end(); )
  {
    iterator r = ri++;
    if (r->second.fromNode()==node || r->second.toNode()==node)
      erase(r);
  }
}

void RouteList::RemoveNodeEventNotify(VrmlNode* node, const char* event)
{
  iterator ri;
  for (ri=begin(); ri!=end(); )
  {
    iterator r = ri++;
    //;!! how about VRML decors?
    if ((r->second.fromNode()==node && strcmp(event, r->second.fromEventOut())==0) ||
        (r->second.toNode()==node && strcmp(event, r->second.toEventIn())==0))
      erase(r);
  }
}

void RouteList::RenameNodeEventNotify(VrmlNode* node, const char* oldEvent, const char* newEvent)
{
  iterator ri;
  for (ri=begin(); ri!=end(); )
  {
    iterator r = ri++;

    //;!! how about VRML decors?

    if (r->second.fromNode()==node && strcmp(oldEvent, r->second.fromEventOut())==0)
      r->second.setFromEventOut(newEvent);

    if (r->second.toNode()==node && strcmp(oldEvent, r->second.toEventIn())==0)
      r->second.setToEventIn(newEvent);
  }
}


void RouteList::RemoveNodeTypeEventNotify(VrmlNodeType* nodeType, const char* event)
{
  iterator ri;
  for (ri=begin(); ri!=end(); )
  {
    iterator r = ri++;
    //;!! how about VRML decors?
    if ((&((r->second.fromNode()->nodeType()))==nodeType && strcmp(event, r->second.fromEventOut())==0) ||
        ((&(r->second.toNode()->nodeType()))==nodeType && strcmp(event, r->second.toEventIn())==0))
      erase(r);
  }
}

void RouteList::RenameNodeTypeEventNotify(VrmlNodeType* nodeType, const char* oldEvent, const char* newEvent)
{
  iterator ri;
  for (ri=begin(); ri!=end(); )
  {
    iterator r = ri++;

    //;!! how about VRML decors?

    if ((&(r->second.fromNode()->nodeType()))==nodeType && strcmp(oldEvent, r->second.fromEventOut())==0)
      r->second.setFromEventOut(newEvent);

    if ((&(r->second.toNode()->nodeType()))==nodeType && strcmp(oldEvent, r->second.toEventIn())==0)
      r->second.setToEventIn(newEvent);
  }
}

void RouteList::Dispatch(double timeStamp, VrmlNode* fromNode, const char* eventOut, const VrmlField &fieldValue)
{
  iterator r;
  for (r=begin(); r!=end(); ++r)
    if (r->second.fromNode()==fromNode && strcmp(eventOut, r->second.fromEventOut())==0)
      r->second.toNode()->eventIn(timeStamp, r->second.toEventIn(), &fieldValue);
}
