/*****************************************************************************
******************************************************************************
*
*               VRML route and route list
*
*               $Revision: 1.1.2.6 $
*               $Date: 2011/08/08 12:08:06 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/

// Copyright 1998-2011 HUMUSOFT s.r.o.
// Portions Copyright (C) 1998  Chris Morley

//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef VRMLROUTE_H
#define VRMLROUTE_H


#include <map>
#include <string>

#include "VrmlNode.h"

class Route : public VrmlObject
{
private:
  friend class RouteList;

  VrmlNode* d_fromNode;
  std::string d_fromEventOut;
  VrmlNode* d_toNode;
  std::string d_toEventIn;

public:
  Route(VrmlNode* fromNode, const char* fromEventOut, VrmlNode* toNode, const char* toEventIn) :
    d_fromNode(fromNode),
    d_fromEventOut(fromEventOut),
    d_toNode(toNode),
    d_toEventIn(toEventIn)
  { }

  // default constructor
  Route() :
    d_fromNode(0),
    d_fromEventOut(0),
    d_toNode(0),
    d_toEventIn(0)
  {
  }

  const char* fromEventOut() const
  { return d_fromEventOut.c_str(); }

  const char* toEventIn() const
  { return d_toEventIn.c_str(); }

  VrmlNode* fromNode() const
  { return d_fromNode; }

  VrmlNode* toNode() const
  { return d_toNode; }


  // Setters
  void setFromNode(VrmlNode* fromNode)
  { d_fromNode = fromNode; }

  void setFromEventOut(const char* fromeventout)
  { d_fromEventOut.assign(fromeventout); }

  void setToNode(VrmlNode* toNode)
  { d_toNode = toNode; }

  void setToEventIn(const char* toeventin)
  { d_toEventIn.assign(toeventin); }

  /**
   * Prints route to stream
   */
  ostream& print(ostream& os, int indent) const;
};


class OPENVRMLDLLEXPORT RouteList : public std::map<int, Route>
{
private:
  int d_idIterator;

public:

  RouteList() : std::map<int, Route>()
  { d_idIterator = 0; }

  /**
   * Adds route to route list
   */
  int Add(VrmlNode* fromNode, const char* fromEventOut, VrmlNode* toNode, const char* toEventIn)
  {
    insert(std::pair<int, Route>(++d_idIterator, Route(fromNode, fromEventOut, toNode, toEventIn)));
    return d_idIterator;
  }

  /**
   * Removes route from route list
   */
  void Remove(VrmlNode* fromNode, const char* fromEventOut, VrmlNode* toNode, const char* toEventIn);

  /**
   * Removes route that reference a given node from route list
   */
  void RemoveNodeNotify(VrmlNode* node);

  /**
   * Removes route which uses a given node event
   */
  void RemoveNodeEventNotify(VrmlNode* node, const char* event);

  /**
   * Node event was renamed, we must rename it in a Route
   */
  void RenameNodeEventNotify(VrmlNode* node, const char* oldEvent, const char* newEvent);

  /**
   * Removes route which uses an event of given node type
   */
  void RemoveNodeTypeEventNotify(VrmlNodeType* nodeType, const char* event);

  /**
   * Node event was renamed, we must rename it in a Route
   */
  void RenameNodeTypeEventNotify(VrmlNodeType* nodeType, const char* oldEvent, const char* newEvent);

  /**
   * Dispatches eventIns for the given eventOut.
   */
  void Dispatch(double timeStamp, VrmlNode* fromNode, const char* eventOut, const VrmlField &fieldValue);
};


#endif // VRMLROUTE_H
