/*****************************************************************************
******************************************************************************
*
*               Wrapper for VrmlNodeType -- it allows to access
*               VRML nodes in VrmlNodeType via VrmlScene pointer.
*
*               $Revision: 1.1.2.3 $
*               $Date: 2011/08/08 12:08:39 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/

// Copyright 1998-2011 HUMUSOFT s.r.o.

#include "VrmlSceneFromNodeType.h"
#include "VrmlNamespace.h"

VrmlSceneFromNodeType::VrmlSceneFromNodeType(VrmlNodeType* type, VrmlScene* parentScene)
  : VrmlScene(type->url())
{
  d_nodeType = type;
  d_parent = parentScene;

  openCount = 1;
}

VrmlSceneFromNodeType::~VrmlSceneFromNodeType()
{
  ///////////////////////////////////////////////////////////////////////
  // VrmlSceneFromNodeType is just wrapper -- do not destruct anything //
  ///////////////////////////////////////////////////////////////////////

  if (openCount) close();
}

// this method is not called in fact
// just made virtual for parity to virtual close() method
void VrmlSceneFromNodeType::open()
{
  // currently does nothing
}

void VrmlSceneFromNodeType::close()
{
  if (openCount <= 0)
    return;

  // scene is destructed -- set scene to NULL
  // so nodes will not access it during their destruction
  if (d_nodeType->nodes())
    d_nodeType->nodes()->addToScene(0,  0);

  // call parent method
  VrmlScene::close();
}

bool VrmlSceneFromNodeType::reload(bool preserveBindings)
{
  return true;
}

VrmlNamespace* VrmlSceneFromNodeType::scope()
{
  return d_nodeType->scope();
}

VrmlNodeGroup* VrmlSceneFromNodeType::nodes()
{
  return d_nodeType->nodes();
}

VrmlNode* VrmlSceneFromNodeType::findNode(const char* name)
{
  if (name == NULL) return NULL;
  return (d_nodeType->scope() != NULL) ? d_nodeType->scope()->findNode(name) : NULL;
}

void VrmlSceneFromNodeType::removeNode(VrmlNode* node)
{
  if (node == NULL) return;

  // reference the node so it doesn't get deleted during RemoveNodeRefs
  node->reference();

  // remove all references to this node in the whole tree
  removeNodeRefs(nodes(), node);

  // remove the node itself
  node->dereference();
}

Doc* VrmlSceneFromNodeType::urlDoc()
{
  Doc* relativeUrl = d_nodeType->getRelativeUrlDoc();
  if (relativeUrl == NULL)
  {
    relativeUrl = d_parent->urlDoc();
  }
  return relativeUrl;
}

VrmlNodeType* VrmlSceneFromNodeType::getNodeType()
{
  return d_nodeType;
}

VrmlScene* VrmlSceneFromNodeType::getParent()
{
  return d_parent;
}

VrmlScene* VrmlSceneFromNodeType::getTopLevelParent()
{
  VrmlScene* topLevelParent = d_parent;
  while (dynamic_cast<VrmlSceneFromNodeType*>(topLevelParent)!=0)
    topLevelParent = dynamic_cast<VrmlSceneFromNodeType*>(topLevelParent)->d_parent;

  return topLevelParent;
}


VrmlScene* VrmlSceneFromNodeType::getClipboardOwnerTopLevelParent()
{
  VrmlScene* topLevelParent = VrmlScene::clipboardOwner;

  while (dynamic_cast<VrmlSceneFromNodeType*>(topLevelParent)!=0)
    topLevelParent = dynamic_cast<VrmlSceneFromNodeType*>(topLevelParent)->d_parent;

  return topLevelParent;
}

bool VrmlSceneFromNodeType::isEdited()
{
  return (getTopLevelParent()->isEdited());
}

void VrmlSceneFromNodeType::setModified()
{ d_nodeType->setModified(); }

void VrmlSceneFromNodeType::clearModified()
{ d_nodeType->clearModified(); }

bool VrmlSceneFromNodeType::isModified() const
{ return d_nodeType->isModified(); }
