/*****************************************************************************
******************************************************************************
*
*               Vrml viewing sector for guessing zfar and znear values
*               (Frustum with very small zfar and big znear)
*
*               $Revision: 1.1.2.1 $
*               $Date: 2009/01/22 16:06:00 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/


// Copyright 1998-2009 HUMUSOFT s.r.o.


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cmath>


#include "MathUtils.h"    // Vcross(), Vnorm(), etc.
#include "VrmlViewingSector.h"

VrmlViewingSector::VrmlViewingSector()
{
  fovy = -1;
  fovx = -1;
  z_far = -1;
  z_near = -1;
}

VrmlViewingSector::VrmlViewingSector(double afovy, double aaspect, Vector3 yawaxis, double yawangle)
{
  fovy = (afovy/360.0)*2.0*M_PI;
  double cy = tan(fovy/2.0);
  fovx = 2.0*atan(cy*aaspect);
  z_near = 0.025; // this value should be enough small
  z_far = 1048576; // 2^20 should have sufficient magnitude
  rotmat = rotmat.setToRotation(yawangle, yawaxis);
  update();
}



void VrmlViewingSector::update()
{
  double cy = tan(fovy/2.0);
  double cx = tan(fovx/2.0);

  double NL = -z_near*cx;
  double NR = -NL;
  double NT =  z_near*cy;
  double NB = -NT;

  float tmp0[3];
  float tmp1[3];

  tmp0[0] = NL;
  tmp0[1] = NB;
  tmp0[2] = -z_near;

  tmp1[0] = NL;
  tmp1[1] = NT;
  tmp1[2] = -z_near;

  Vcross(left_plane, tmp0, tmp1);
  Vnorm(left_plane);
  left_plane[3] = 0;

  right_plane[0] = -left_plane[0];
  right_plane[1] = 0;
  right_plane[2] = left_plane[2];
  right_plane[3] = 0;

  tmp0[0] = NL;
  tmp0[1] = NT;
  tmp0[2] = -z_near;

  tmp1[0] = NR;
  tmp1[1] = NT;
  tmp1[2] = -z_near;

  Vcross(top_plane, tmp0, tmp1);
  Vnorm(top_plane);
  top_plane[3] = 0;

  bot_plane[0] = 0;
  bot_plane[1] = -top_plane[1];
  bot_plane[2] = top_plane[2];
  bot_plane[3] = 0;
}

bool VrmlViewingSector::isInSector(Vector3 position) const
{
  double x = position.getX();
  double y = position.getY();
  double z = position.getZ();

  Point3 left = Point3(left_plane[0], left_plane[1], left_plane[2]);
  Point3 right = Point3(right_plane[0], right_plane[1], right_plane[2]);
  Point3 top = Point3(top_plane[0], top_plane[1], top_plane[2]);
  Point3 bot = Point3(bot_plane[0], bot_plane[1], bot_plane[2]);

  left = rotmat * left;
  right = rotmat * right;
  top = rotmat * top;
  bot = rotmat * bot;

  Vector3 leftVec = Vector3(left.getX(), left.getY(), left.getZ());
  Vector3 rightVec = Vector3(right.getX(), right.getY(), right.getZ());
  Vector3 topVec = Vector3(top.getX(), top.getY(), top.getZ());
  Vector3 botVec = Vector3(bot.getX(), bot.getY(), bot.getZ());

  double d_bot = botVec.dot(position);
  double d_top = topVec.dot(position);
  double d_left = leftVec.dot(position);
  double d_right = rightVec.dot(position);

  bool isInSector = ((d_bot >= 0) && (d_top >= 0) && (d_left >= 0) && (d_right >= 0) && (std::abs(z) - z_near >= 0) && (std::abs(z) - z_far <= 0));
  return isInSector;
}
