/*****************************************************************************
******************************************************************************
*
*               Vrml viewing sector for guessing zfar and znear values
*               (Frustum with very small zfar and big znear)
*
*               $Revision: 1.1.2.1 $
*               $Date: 2009/01/22 16:12:17 $
*               $Author: Xjhouska $
*
******************************************************************************
*****************************************************************************/


// Copyright 1998-2009 HUMUSOFT s.r.o.


#ifndef VRMLVIEWINGSECTOR_H
#define VRMLVIEWINGSECTOR_H

#include <iostream>
#include "Math3d.h"

class VrmlViewingSector
{
private:
  /**
   * Vertical field of view.
   */
  double fovy;

  /**
   * Horizontal field of view.
   */
  double fovx;

  /**
   * Distance to the near clipping plane.
   */
  double z_near;

  /**
   * Distance to the far clipping plane
   */
  double z_far;

  /**
   * Left (looking down -z) side clip plane. Format is (a,b,c,d) where
   * (a,b,c) is the plane normal and d is the plane offset. For the
   * movement the eyepoint is always the origin, so d is going to be
   * 0.
   */
  float left_plane[4];
  float right_plane[4];
  float top_plane[4];
  float bot_plane[4];

  /**
   * rotation matrix defining viewing sector rotation betwen current and default view (up vector (0,1,0) and right vector (1,0,0))
   */
  Matrix4x4 rotmat;


public:
  VrmlViewingSector();
  VrmlViewingSector(double afovy, double aaspect, Vector3 yawaxis, double yawangle);
  bool isInSector(Vector3 position) const;
  void update();

};



#endif // VRMLVIEWINGSECTOR_H
